# -*- coding: utf-8 -*-
## PloneSubscription
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Makes document subscription catalog aware.
"""
__version__ = "$Revision: 1.6 $"
# $Source: /cvsroot/ingeniweb/PloneSubscription/SubscriptionCatalogAware.py,v $
# $Id: SubscriptionCatalogAware.py,v 1.6 2005/09/22 12:56:30 encolpe Exp $
__docformat__ = 'restructuredtext'


# Python imports
from AccessControl import ClassSecurityInfo
from AccessControl.PermissionRole import rolesForPermissionOn

# CMF imports
from Products.CMFCore.utils import getToolByName
from zLOG import LOG, INFO

# Archetypes imports
from Products.Archetypes.public import *

# Products imports
from Products.PloneSubscription.config import PROJECTNAME, TOOL_ID
from Products.PloneSubscription import SubscriptionPermissions

class SubscriptionCatalogAwareContent(BaseContent):
    """Base class to be used for being indexed in subscriptions catalog"""

    security = ClassSecurityInfo()

    def _catalogSubscription(self):
        """Catalog vocabulary"""

        catalog = getToolByName(self, TOOL_ID).getCatalog()
        if catalog is not None:
            url = self._getURL()
            catalog.catalog_object(self, url)
        else:
            LOG("WARNING /!\\", INFO, "No subscription catalog")

    def _uncatalogSubscription(self):
        """UnCatalog vocabulary"""

        catalog = getToolByName(self, TOOL_ID).getCatalog()
        if catalog is not None:
            url = self._getURL()
            catalog.uncatalog_object(url)
        else:
            LOG("WARNING /!\\", INFO, "No subscription catalog")

    # Overload AT catalog methods.
    def indexObject(self):
        #self._catalogSubscription()
        catalog = getToolByName(self, TOOL_ID).getCatalog()
        if catalog is not None:
            catalog.indexObject(self)

        BaseContent.indexObject(self)

    def unindexObject(self):
        #self._uncatalogSubscription()
        catalog = getToolByName(self, TOOL_ID).getCatalog()
        if catalog is not None:
            catalog.unindexObject(self)

        BaseContent.unindexObject(self, *args, **kwargs)

    def reindexObject(self, idxs=[]):
        #self._catalogSubscription()
        catalog = getToolByName(self, TOOL_ID).getCatalog()
        if catalog is not None:
            catalog.reindexObject(self, idxs=idxs)

        BaseContent.reindexObject(self, idxs=idxs)

    security.declareProtected(SubscriptionPermissions.EditSubscriptionContent, 'reindexObjectSecurity')
    def reindexObjectSecurity(self):
        """
            Reindex security-related indexes on the object
            (and its descendants).
        """
        catalogs = (getToolByName(self, 'portal_catalog', None),
                    getToolByName(self, TOOL_ID).getCatalog())
        for catalog in catalogs:
            if catalog is not None:
                path = '/'.join(self.getPhysicalPath())
                for brain in catalog.searchResults(path=path):
                    ob = brain.getObject()
                    if ob is None:
                        # Ignore old references to deleted objects.
                        continue
                    s = getattr(ob, '_p_changed', 0)
                    catalog.reindexObject(ob, idxs=['allowedRolesAndUsers'])
                    if s is None: ob._p_deactivate()
                # Reindex the object itself, as the PathIndex only gave us
                # the descendants.
                self.reindexObject(idxs=['allowedRolesAndUsers'])
