# -*- coding: iso-8859-1 -*-
#
# $Id: ConfigletsInstaller.py,v 1.1 2005/10/23 21:52:51 roeder Exp $
#
#
# ***********************************************************
# ** (Un)installs the configlets for the portal config PMI **
# ***********************************************************
#
# Example:
#
#     ...
#     # Shortest possible
#     myConfiglet1 = {
#         'id': 'my_configlet',
#         'name': "Configure the features of my product",
#         'action': 'string:${portal_url}/prefs_my_product_form1'}
#     ci1 = ConfigletsInstaller(myConfiglet1)
#
#     # Several configlets
#     myConfiglets2 = (
#         {'id': 'my_configlet2',
#          'name': "Configure second group of features of my product",
#          'action': 'string:${portal_url}/prefs_my_product_form2',
#          'condition': "python: user.has_role('Manager')"},
#         {'id': 'my_configlet3',
#          'name': "Configure third group of features of my product",
#          'action': 'string:${portal_url}/prefs_my_product_form3',
#          'condition': "python: foo == bar",
#          'imageUrl': 'someIcon.gif'}
#         )
#     ci2 = ConfigletsInstaller(myConfiglets2)
#     ...
#

__author__  = 'Gilles Lenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

from Products.CMFCore.CMFCorePermissions import ManagePortal

from utils import *

__all__ = ('ConfigletsInstaller',)

class ConfigletsInstaller(InstallerBase):

    _installerTitle = "configlet in portal configuration"


    def __init__(self, configlets, **kw):
        """Constructor
        @param configlet: (sequence of) mappings with keys:
            o * id: the id of the configlet
            o * name: a label for your configlet
            o * action: TALES expression
            o condition: TALES expression (default: empty expression)
            o permission: defaults to ManagePortal
            o category: defaults to 'Products'
            o visible: default is 1 (visible)
            o appId: defaults is the product's name
            o imageUrl: defaults ton None (no image)
            o description: defaults to empty string
            keys starting with '*' are mandatory

        @param kw: override defaults like stop_on_error attribute
        """
        if type(configlets) == type({}):
            configlets = [configlets]
        self.configlets = tuple(configlets)
        for k, v in kw.items():
            setattr(self, k, v)
        return


    def doInstall(self, context):
        """Adds configlets to the portal configuration utility
        """
        # building the default values for not provided keys
        defaults = {
            'permission': ManagePortal,
            'category': 'Products',
            'appId': context.productName()}
        for configlet in self.configlets:
            for k, v in defaults.items():
                if not configlet.has_key(k):
                    configlet[k] = v
            # FIXME: should check existing configlets first!
            context.portal_controlpanel.registerConfiglet(**configlet)
            context.logInfo("Added configlet '%s' to portal control panel" %
                            configlet['name'])
        return


    def doUninstall(self, context):
        """Uninstall of configlets is not supported by CMFQuickInstaller
        @param context: an InstallationContext object
        """
        for configlet in self.configlets:
            # FIXME: should check existing configlets and log problems
            context.portal_controlpanel.unregisterConfiglet(configlet['id'])
            context.logInfo("Removed configlet '%s' from portal control panel" %
                            configlet['name'])
        return
