# -*- coding: iso-8859-1 -*-
#
# $Id: StandardPropertiesInstaller.py,v 1.1 2005/10/23 21:52:51 roeder Exp $
#
# *****************************************************************************
# ** Changes the value of properties from the Plone standard property sheets
# **
# *****************************************************************************
#
# Example:
#
#     ...
#     ntPropItems = (
#         ('batchSize', 40),
#         ('idsNotToList', ['secret', 'confidential']))
#     spi = StandardPropertiesInstaller('navtree_properties', propItems)
#     ...
#
# FIXME: There's no easy way to "unchange" what has been done here.
#

__author__  = 'Gilles Lenfant <gilles.lenfant@ingeniweb.com>'
__docformat__ = 'restructuredtext'

from utils import *

__all__ = ('StandardPropertiesInstaller',)

class StandardPropertiesInstaller(InstallerBase):

    _installerTitle = "new values for standard properties"


    def __init__(self, sheetName, properties, **kw):
        """Constructor
        @param sheetName: the name as appearing in portal_properties
        @param properties: a sequence of (id, value, type) tuples
        @param kw: override defaults like stop_on_error attribute
        """
        self.sheetName = sheetName
        self.properties = properties
        for k, v in kw.items():
            setattr(self, k, v)
        return


    def doInstall(self, context):
        """Changes existing properties
        @param context: an InstallationContext object
        """
        propertySheet = getattr(context.portal_properties, self.sheetName)
        for property in self.properties:
            id = property[0]
            value = property[1]
            tipe = None
            
            if len(property) > 2:
                tipe = property[2]
            
            if propertySheet.hasProperty(id) or tipe is None:
                propertySheet._setPropValue(id, value)
            else:
                propertySheet._setProperty(id, value, tipe)
            
            context.logInfo("Property '%s' set to %s in property sheet '%s'" %
                            (id, str(value), self.sheetName))
        return


    def doUninstall(self, context):
        """Can't undo changes
        @param context: an InstallationContext object
        """
        propIds = ' ,'.join(["'" + p[0].strip() + "'" for p in self.properties])
        context.logWarning("Can't reset the previous property values for %s in '%s' property sheet" %
                           (propIds, self.sheetName))
        return
